/* Initialize some Express stuff, including the domain
   decomposition of the lattice across the processors.
*/

#include "comext.h"
#include <stdio.h>
#include <math.h>

decompose()
{
   int i, clustmax, proc_array_num;
   int d1,d2,d3,type;

   fasync(stderr);
   
   /* Set up node data */
   exparam(&nodedata);
   procnum = nodedata.procnum;
   nprocs = nodedata.nprocs;

   /* Domain decomposition using gridmap tools */

   /* Get the dimensions of the 2-d processor array in procdim */
   exgridsplit(nprocs,2,procdim);
   d1 = exgridinit(2,procdim);

   /* Get the coords of this processor in the proc grid */
   d2 = exgridcoord(procnum,proccoord);

   /* Get the coords of neighboring processors */
   left   = exgridnode(procnum, 0, -1);
   right  = exgridnode(procnum, 0,  1);
   bottom = exgridnode(procnum, 1, -1);
   top    = exgridnode(procnum, 1,  1);

   /* Actual position of the proc in the proc array */
   proc_array_num = (proccoord[1] * procdim[0]) + proccoord[0];

#ifdef DEBUG
forder(stdout,proc_array_num);
fmulti(stdout);
printf("procnum = %d  left = %d  right = %d  bottom = %d  top = %d\n",
	  procnum,left,right,bottom,top);
fflush(stdout);
fsingl(stdout);
#endif

   /* Decompose the global arrays onto processors */
   globsize[0] = globisiz;
   globsize[1] = globjsiz;
   d3 = exgridsize(procnum,globsize,localsiz,orig);
   isiz = localsiz[0];
   jsiz = localsiz[1];
   sites = isiz * jsiz;

   /* Sizes for edges to be passed */
   isiz_int = isiz * sizeof(int);
   jsiz_int = jsiz * sizeof(int);
   isiz_short = isiz * sizeof(short);
   jsiz_short = jsiz * sizeof(short);
   jmax_short = JMAX * sizeof(short);
   jmax_int = JMAX * sizeof(int);
   two_isiz_int = 2 * isiz * sizeof(int);
   two_jsiz_int = 2 * jsiz * sizeof(int);
   /*
   max_isiz = isiz + (isiz + 1) / 4;
   max_jsiz = jsiz + (jsiz + 1) / 4;
   */
   max_isiz = isiz + (isiz/4)  + ((isiz%4) != 0);
   max_jsiz = jsiz + (jsiz/4)  + ((jsiz%4) != 0);
   max_isiz_int = max_isiz * sizeof(int);
   max_jsiz_int = max_jsiz * sizeof(int);

   /* Segment a word into four bytes */
   for (i=0;i<4;i++) shiftsize[i] = i * SIZE_OF_BYTE;

   /* Offset for cluster labels */
   /* offset = sites * procnum; */
   if (sites < 64) clustmax = sites;
   else clustmax = sites/4;
#ifdef DEBUG
   offset = clustmax * proc_array_num;
#else
   offset = clustmax * procnum;
#endif

   /* Minimum number of iterations before checking if something changed
      in the self_label subroutine. This should go like sqrt(nprocs),
      with some tunable multiplicative constant between 1 and 2 */
   minits = 1.8 * sqrt((double)nprocs);
   printf("minits in self_labeling = %d\n", minits);

   /* DEBUG */
   if ((d1 < 0) || (d2 < 0) || (d3 < 0)) {
     fprintf(stderr,"exgridmap %d  %d  %d for proc %d\n", d1,d2,d3,procnum);
     fflush(stderr);
     abort(-3);
   }

   if ((isiz > (IMAX-2)) || (jsiz > (JMAX-2))) {
      perror("array size too large");
      exit(-3);
   }
   else {
      printf("Using a %d x %d processor array\n",procdim[0],procdim[1]);
      printf("with a %d x %d lattice segment on each processor.\n",isiz,jsiz);
      fflush(stdout);
   }

#ifdef DEBUG
   fmulti(stdout);
   printf("orig = %d %d\n", orig[0],orig[1]);
   fflush(stdout);
   fsingl(stdout);
#endif

#ifdef GRAPHICS
        openpl(16000, NULL);
	rainbow(0,255);
 
	x0 = (double)proccoord[0]/(double)procdim[0];
	y0 = (double)proccoord[1]/(double)procdim[1];
	vport(x0,y0,x0+1./(double)procdim[0],y0+1./(double)procdim[1]);
        space(0.0, 0.0, (double)(isiz), (double)(jsiz));
        setclip(0.0, 0.0, (double)(isiz), (double)(jsiz));
        color(1);
        move(0., 0.);cont(0.,(double)jsiz);cont((double)isiz, (double)jsiz);
        cont((double)isiz, 0.0);cont(0.,0.);
  usendplot();
#endif

}

